/**
 * @module "@blitzm/kubernetes-nginx-ingress"
 * @packageDocumentation
 * @example
 * ```typescript
 * import { KubernetesNginxIngress } from '@blitzm/kubernetes-nginx-ingress';
 *
 * new KubernetesNginxIngress('my-thing', {
 *   // ...
 * });
 * ```
 */
import * as pulumi from '@pulumi/pulumi';
import * as k8s from '@pulumi/kubernetes';
export interface KubernetesNginxIngressInputs {
    provider?: k8s.Provider;
    namespace?: pulumi.Input<string>;
    /**
     * the helm chart version
     */
    version?: string;
    /**
     * mode configures nginx as either a kubernetes
     * deployment or daemonset.
     *
     * In Deployment mode a kubernetes Deployment will be
     * created behind a kubernetes Service of type LoadBalancer.
     * The Deployment mode is intended to be used in Cloud Environments.
     *
     * In DaemonSet mode a kubernetes DaemonSet will be created
     * without a kubernetes Service, instead, hostPorts will be
     * used by the DaemonSet containers. The DaemonSet mode is intended
     * to be used in onpremise environments where a LoadBalancer services
     * may not available.
     *
     * defaults to Deployment mode.
     */
    mode?: NginxIngressDeploymentMode | NginxIngressDaemonSetMode;
    /**
     * config sets nginx config map entries.
     *
     * see: https://github.com/kubernetes/ingress-nginx/blob/master/docs/user-guide/nginx-configuration/configmap.md
     */
    config?: Record<string, string>;
    /**
     * configure any L4 TCP services
     */
    tcpServices?: Record<number, L4ServiceBackend>;
    /**
     * configure any L4 UDP services
     */
    udpServices?: Record<number, L4ServiceBackend>;
    /**
     * maxmind license key to download GeoLite2 Databases
     * https://blog.maxmind.com/2019/12/18/significant-changes-to-accessing-and-using-geolite2-databases
     */
    maxmindLicenseKey?: string;
    /**
     * Enables the SSL Passthrough feature in nginx-ingress.
     * This is required to use the `nginx.ingress.kubernetes.io/ssl-passthrough`
     * annotation for SNI based virtual domain (host) routing.
     *
     * https://kubernetes.github.io/ingress-nginx/user-guide/nginx-configuration/annotations/#ssl-passthrough
     *
     * https://kubernetes.github.io/ingress-nginx/user-guide/tls/#ssl-passthrough
     */
    enableSSLPassthrough?: boolean;
}
export interface NginxIngressDeploymentMode {
    kind: 'Deployment';
    /**
     * how many nginx ingress controller
     * replicas should be deployed.
     *
     * defaults to 1
     */
    replicas?: pulumi.Input<number>;
    /**
     * loadBalancerIP configures the cloud loadbalancer
     * IP address on supported clouds.
     * This IP will need to be provisioned using the
     * appropriate cloud resource.
     *
     * If this in left blank then kubernetes will assign
     * an IP address to the nginx-ingress LoadBalancer Service
     * for you (if supported by your cloud provider).
     */
    loadBalancerIP?: pulumi.Input<string>;
    /**
     * serviceAnnotations adds the given annotations to the
     * nginx-ingress loadbalancer service
     */
    serviceAnnotations?: pulumi.Input<Record<string, pulumi.Input<string>>>;
}
export interface NginxIngressDaemonSetMode {
    kind: 'DaemonSet';
}
export interface L4ServiceBackend {
    namespace: pulumi.Input<string>;
    serviceName: pulumi.Input<string>;
    servicePort: pulumi.Input<number>;
}
export interface KubernetesNginxIngressOutputs {
    /**
     * the 'kubernetes.io/ingress.class' annotation that this
     * ingress controller will consume
     */
    ingressClass: pulumi.Output<string>;
    /**
     * The kubernetes service resource of the ingress controller
     */
    ingressService: pulumi.Output<k8s.core.v1.Service>;
}
/**
 * @noInheritDoc
 */
export declare class KubernetesNginxIngress extends pulumi.ComponentResource implements KubernetesNginxIngressOutputs {
    readonly ingressClass: pulumi.Output<string>;
    readonly ingressService: pulumi.Output<k8s.core.v1.Service>;
    constructor(name: string, props?: KubernetesNginxIngressInputs, opts?: pulumi.CustomResourceOptions);
    private controllerValues;
    private daemonSetValues;
    private deploymentValues;
    private l4Services;
    private extraArgs;
}
